@@ -29,8 +29,8 @@ def construct(plot, view, w, h):
         else:
             plot.original_points = copy.copy(plot.points)
             plot.original_lines = copy.copy(plot.lines)
-        for i in plot.late_vertexors:
-            i(view, w, h)
+        for vertexor, x, y, r, g, b, a in plot.late_vertexors:
+            vertexor(plot, view, w, h, x, y, r, g, b, a)
     # points
     for x, y, r, g, b, a in plot.points:
         plot.buffer.add(x, y, *fcolor(r, g, b, a))
@@ -75,8 +75,17 @@ def show(plot, w, h):
     def reset():
         view.x = plot.x_min
         view.y = plot.y_min
-        view.w = plot.x_max-plot.x_min
-        view.h = plot.y_max-plot.y_min
+        view.w = plot.x_max - plot.x_min
+        view.h = plot.y_max - plot.y_min
+        if view.x == math.inf:
+            view.x = 0
+            view.y = 0
+            view.w = 1
+            view.h = 1
+        if view.w == 0:
+            view.w = 1
+        if view.h == 0:
+            view.h = 1
         media.view_set(*view.tuple())
     def move(view, dx, dy):
         view.x -= dx*view.w/media.width()
@@ -164,7 +173,16 @@ def show(plot, w, h):
             while i < view.x + view.w:
                 s = '{:.5}'.format(i)
                 if view.x + view.w - i > increment:
-                    texter.text(s, x=i+margin_x, y=view.y+margin_y, w=text_w, h=text_h)
+                    if i == 0 and plot.epochs.get(0) != None:
+                        texter.text(
+                            plot.epochs[0].isoformat('\n'),
+                            x=i + margin_x,
+                            y=view.y + margin_y + text_h,
+                            w=text_w / 2,
+                            h=text_h / 2,
+                        )
+                    else:
+                        texter.text(s, x=i+margin_x, y=view.y+margin_y, w=text_w, h=text_h)
                     texter.text('L', i, view.y, text_w * 2, text_h)
                 i += increment
             # draw y axis
@@ -176,7 +194,16 @@ def show(plot, w, h):
             i = (view.y + text_h + 2*margin_y) // increment * increment + increment
             while i < view.y + view.h - (text_h + 2*margin_y):
                 s = '{:.5}'.format(i)
-                texter.text(s, x=view.x+margin_x, y=i+margin_y, w=text_w, h=text_h)
+                if i == 0 and plot.epochs.get(1) != None:
+                    texter.text(
+                        plot.epochs[1].isoformat('\n'),
+                        x=view.x + margin_x,
+                        y=i + margin_y + text_h,
+                        w=text_w / 2,
+                        h=text_h / 2,
+                    )
+                else:
+                    texter.text(s, x=view.x+margin_x, y=i+margin_y, w=text_w, h=text_h)
                 texter.text('L', view.x, i, text_w * 2, text_h)
                 i += increment
         plot.buffer_text.data = texter.data
